import java.util.*;
import javax.microedition.lcdui.*;

public abstract class CalendarTableClass extends Canvas {
    private static final int COLOR_BACKGROUND          = 0xFFFFFF;
    private static final int COLOR_LABEL_TEXT          = 0x000000;
    private static final int COLOR_BORDER              = 0x006699;
    private static final int COLOR_HEAD_TEXT           = 0xFFFFFF;
    private static final int COLOR_HEAD_BACKGROUND     = 0x0099CC;
    private static final int COLOR_BODY_TEXT           = 0x000000;
    private static final int COLOR_BODY_BACKGROUND     = 0xFFFFFF;
    private static final int COLOR_SEL_BODY_TEXT       = 0x000000;
    private static final int COLOR_SEL_BODY_BACKGROUND = 0xCCCCCC;

    private static final int SLIDE_DELTA_X = 20;
    private static final int SLIDE_DELTA_Y = 40;

    public static final int ITEM_ALIGN_LEFT  = 1;
    public static final int ITEM_ALIGN_RIGHT = 2;

    private static final String NO_DATA_MSG = "No data";

    private int    Align[], FloatColNum, Cols, LabelHeight, TitleHeight, RowHeight, BottomHeight, VisibleRows, FirstVisibleRow,
                   PointerPressedX = -1, PointerPressedY = -1;
    private String Label, Title[];

    private Font CellFont = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_SMALL);

    protected Calendar Month;
    protected int      Rows, SelectedRow;
    protected String   Data[][];

    protected CalendarTableClass(String screen_title, String label, Calendar month, String title[], int align[], int float_col_num) {
        super();

        setTitle(screen_title);

        Label           = label;
        Month           = month;
        Title           = title;
        Align           = align;
        FloatColNum     = float_col_num;
        Cols            = Title.length;
        RowHeight       = CellFont.getHeight() + 2;
        LabelHeight     = RowHeight;
        TitleHeight     = RowHeight;
        BottomHeight    = RowHeight;
        VisibleRows     = 0;
        FirstVisibleRow = 0;
        Rows            = 0;
        SelectedRow     = 0;
        Data            = null;
    }

    private void SetNextMonth() {
        int month = Month.get(Calendar.MONTH);

        if (month == Calendar.DECEMBER) {
            Month.set(Calendar.YEAR, Month.get(Calendar.YEAR) + 1);
            Month.set(Calendar.MONTH, Calendar.JANUARY);
        } else {
            Month.set(Calendar.MONTH, month + 1);
        }
    }

    private void SetPrevMonth() {
        int month = Month.get(Calendar.MONTH);

        if (month == Calendar.JANUARY) {
            Month.set(Calendar.YEAR, Month.get(Calendar.YEAR) - 1);
            Month.set(Calendar.MONTH, Calendar.DECEMBER);
        } else {
            Month.set(Calendar.MONTH, month - 1);
        }
    }

    protected int GetCols() {
        return Cols;
    }

    protected void SetColumnTitle(int column, String title) {
        Title[column] = title;
    }

    protected void SetLabel(String label) {
        Label = label;
    }

    protected abstract void RefreshData();

    protected void keyPressed(int key_code) {
        int game_action = getGameAction(key_code);

        if (game_action == UP) {
            if (SelectedRow > 0) {
                SelectedRow--;
            } else {
                SelectedRow = Rows - 1;
            }

            repaint();
        } else if (game_action == DOWN) {
            if (SelectedRow < (Rows - 1)) {
                SelectedRow++;
            } else {
                SelectedRow = 0;
            }

            repaint();
        } else if (game_action == LEFT) {
            SetPrevMonth();
            RefreshData();

            repaint();
        } else if (game_action == RIGHT) {
            SetNextMonth();
            RefreshData();

            repaint();
        }
    }

    protected void keyRepeated(int key_code) {
        int game_action = getGameAction(key_code);

        if (game_action == UP) {
            if (SelectedRow > 0) {
                SelectedRow--;
            } else {
                SelectedRow = Rows - 1;
            }

            repaint();
        } else if (game_action == DOWN) {
            if (SelectedRow < (Rows - 1)) {
                SelectedRow++;
            } else {
                SelectedRow = 0;
            }

            repaint();
        } else if (game_action == LEFT) {
            SetPrevMonth();
            RefreshData();

            repaint();
        } else if (game_action == RIGHT) {
            SetNextMonth();
            RefreshData();

            repaint();
        }
    }

    protected void pointerPressed(int x, int y) {
        if (y < LabelHeight + TitleHeight) {
            if (SelectedRow > 0) {
                SelectedRow--;

                repaint();
            }
        } else if (y > LabelHeight + TitleHeight + VisibleRows * RowHeight) {
            if (SelectedRow < (Rows - 1)) {
                SelectedRow++;

                repaint();
            }
        } else {
            int selected_row = FirstVisibleRow + (y - LabelHeight - TitleHeight) / RowHeight;

            if (selected_row >= FirstVisibleRow && selected_row < FirstVisibleRow + VisibleRows) {
                SelectedRow = selected_row;

                repaint();
            }
        }
        PointerPressedX = x;
        PointerPressedY = y;
    }

    protected void pointerReleased(int x, int y) {
        if (PointerPressedX != -1 && PointerPressedY != -1) {
            if ((PointerPressedY < y && y - PointerPressedY < SLIDE_DELTA_Y) ||
                (PointerPressedY > y && PointerPressedY - y < SLIDE_DELTA_Y)) {
                if (PointerPressedX < x && x - PointerPressedX > SLIDE_DELTA_X) {
                    SetPrevMonth();
                    RefreshData();

                    repaint();
                } else if (PointerPressedX > x && PointerPressedX - x > SLIDE_DELTA_X) {
                    SetNextMonth();
                    RefreshData();

                    repaint();
                }
            }
            PointerPressedX = -1;
            PointerPressedY = -1;
        }
    }

    protected void pointerDragged(int x, int y) {
        if (y < LabelHeight + TitleHeight) {
            if (SelectedRow > 0) {
                SelectedRow--;

                repaint();
            }
        } else if (y > LabelHeight + TitleHeight + VisibleRows * RowHeight) {
            if (SelectedRow < (Rows - 1)) {
                SelectedRow++;

                repaint();
            }
        } else {
            int selected_row = FirstVisibleRow + (y - LabelHeight - TitleHeight) / RowHeight;

            if (selected_row >= FirstVisibleRow && selected_row < FirstVisibleRow + VisibleRows) {
                SelectedRow = selected_row;

                repaint();
            }
        }
    }

    protected void paint(Graphics graphics) {
        int col_width[] = new int[Cols];

        RowHeight    = CellFont.getHeight() + 2;
        LabelHeight  = RowHeight;
        TitleHeight  = RowHeight;
        BottomHeight = RowHeight;

        for (int i = 0; i < Cols; i++) {
            col_width[i] = 0;
        }
        for (int i = 0; i < Cols; i++) {
            if (i != FloatColNum && Title[i] != null) {
                int cell_width = CellFont.stringWidth(Title[i]);

                if (col_width[i] < cell_width) {
                    col_width[i] = cell_width + 2;
                }
            }
        }
        for (int i = 0; i < Rows; i++) {
            for (int j = 0; j < Cols; j++) {
                if (j != FloatColNum && Data[i][j] != null) {
                    int cell_width = CellFont.stringWidth(Data[i][j]);

                    if (col_width[j] < cell_width) {
                        col_width[j] = cell_width + 2;
                    }
                }
            }
        }

        if (FloatColNum != -1) {
            int total_width = 0;

            for (int i = 0; i < Cols; i++) {
                if (i != FloatColNum) {
                    total_width = total_width + col_width[i];
                }
            }

            col_width[FloatColNum] = getWidth() - total_width;

            if (col_width[FloatColNum] < 0) {
                col_width[FloatColNum] = 0;
            }
        }

        VisibleRows = (getHeight() - LabelHeight - TitleHeight - BottomHeight) / RowHeight;

        if (VisibleRows > Rows) {
            VisibleRows = Rows;
        }

        if (FirstVisibleRow > Rows - VisibleRows) {
            FirstVisibleRow = Rows - VisibleRows;
        }
        if (SelectedRow < FirstVisibleRow) {
            FirstVisibleRow = SelectedRow;
        } else if (SelectedRow >= FirstVisibleRow + VisibleRows) {
            FirstVisibleRow = SelectedRow - VisibleRows + 1;
        }

        int last_visible_row = FirstVisibleRow + VisibleRows - 1;

        if (last_visible_row >= Rows) {
            last_visible_row = Rows - 1;
        }

        graphics.setFont(CellFont);

        graphics.setColor(COLOR_BACKGROUND);
        graphics.fillRect(0, 0, getWidth(), getHeight());

        int curr_x = 0;
        int curr_y = 0;

        graphics.setColor(COLOR_LABEL_TEXT);

        if (Label != null) {
            int old_clip_x = graphics.getClipX();
            int old_clip_y = graphics.getClipY();
            int old_clip_w = graphics.getClipWidth();
            int old_clip_h = graphics.getClipHeight();

            graphics.setClip(curr_x + 1, curr_y + 1, getWidth() - 1, LabelHeight - 1);

            graphics.drawString(Label, curr_x + 2, curr_y + LabelHeight - 1, Graphics.BOTTOM | Graphics.LEFT);

            graphics.setClip(old_clip_x, old_clip_y, old_clip_w, old_clip_h);
        }

        graphics.setColor(COLOR_BORDER);

        curr_x = 0;
        curr_y = LabelHeight;

        for (int i = 0; i <= VisibleRows + 1; i++) {
            graphics.drawLine(0, curr_y, getWidth(), curr_y);

            curr_y = curr_y + RowHeight;
        }

        curr_x = 0;
        curr_y = LabelHeight;

        for (int i = 0; i <= Cols; i++) {
            if (i > 0 && i < Cols) {
                curr_x = curr_x + col_width[i - 1];
            } else if (i == Cols) {
                curr_x = getWidth();
            }
            graphics.drawLine(curr_x, curr_y, curr_x, TitleHeight + (VisibleRows + 1) * RowHeight);
        }

        curr_x = 0;
        curr_y = LabelHeight;

        for (int i = 0; i < Cols; i++) {
            graphics.setColor(COLOR_HEAD_BACKGROUND);
            graphics.fillRect(curr_x + 1, curr_y + 1, col_width[i] - 1, TitleHeight - 1);
            graphics.setColor(COLOR_HEAD_TEXT);

            if (Title[i] != null) {
                int old_clip_x = graphics.getClipX();
                int old_clip_y = graphics.getClipY();
                int old_clip_w = graphics.getClipWidth();
                int old_clip_h = graphics.getClipHeight();

                graphics.setClip(curr_x + 1, curr_y + 1, col_width[i] - 1, TitleHeight - 1);

                if (Align[i] == ITEM_ALIGN_LEFT) {
                    graphics.drawString(Title[i], curr_x + 2, curr_y + TitleHeight - 1, Graphics.BOTTOM | Graphics.LEFT);
                } else if (Align[i] == ITEM_ALIGN_RIGHT) {
                    graphics.drawString(Title[i], curr_x + col_width[i] - CellFont.stringWidth(Title[i]),
                                       curr_y + TitleHeight - 1, Graphics.BOTTOM | Graphics.LEFT);
                }

                graphics.setClip(old_clip_x, old_clip_y, old_clip_w, old_clip_h);

                if (i < Cols - 1) {
                    curr_x = curr_x + col_width[i];
                } else {
                    curr_x = getWidth();
                }
            }
        }

        curr_y = LabelHeight + TitleHeight;

        for (int i = FirstVisibleRow; i <= last_visible_row ; i++) {
            curr_x = 0;

            for (int j = 0; j < Cols; j++) {
                if (i == SelectedRow) {
                    graphics.setColor(COLOR_SEL_BODY_BACKGROUND);
                    graphics.fillRect(curr_x + 1, curr_y + 1, col_width[j] - 1, RowHeight - 1);
                    graphics.setColor(COLOR_SEL_BODY_TEXT);
                } else {
                    graphics.setColor(COLOR_BODY_BACKGROUND);
                    graphics.fillRect(curr_x + 1, curr_y + 1, col_width[j] - 1, RowHeight - 1);
                    graphics.setColor(COLOR_BODY_TEXT);
                }

                if (Data[i][j] != null) {
                    int old_clip_x = graphics.getClipX();
                    int old_clip_y = graphics.getClipY();
                    int old_clip_w = graphics.getClipWidth();
                    int old_clip_h = graphics.getClipHeight();

                    graphics.setClip(curr_x + 1, curr_y + 1, col_width[j] - 1, RowHeight - 1);

                    if (Align[j] == ITEM_ALIGN_LEFT) {
                        graphics.drawString(Data[i][j], curr_x + 2, curr_y + RowHeight - 1, Graphics.BOTTOM | Graphics.LEFT);
                    } else if (Align[j] == ITEM_ALIGN_RIGHT) {
                        graphics.drawString(Data[i][j], curr_x + col_width[j] - CellFont.stringWidth(Data[i][j]),
                                           curr_y + RowHeight - 1, Graphics.BOTTOM | Graphics.LEFT);
                    }

                    graphics.setClip(old_clip_x, old_clip_y, old_clip_w, old_clip_h);
                }

                if (j < Cols - 1) {
                    curr_x = curr_x + col_width[j];
                } else {
                    curr_x = getWidth();
                }
            }

            curr_y = curr_y + RowHeight;
        }

        if (Rows == 0) {
            graphics.setColor(COLOR_BODY_TEXT);

            int old_clip_x = graphics.getClipX();
            int old_clip_y = graphics.getClipY();
            int old_clip_w = graphics.getClipWidth();
            int old_clip_h = graphics.getClipHeight();

            graphics.setClip((getWidth() - CellFont.stringWidth(NO_DATA_MSG)) / 2, (getHeight() - LabelHeight - TitleHeight) / 2,
                              CellFont.stringWidth(NO_DATA_MSG), RowHeight);
            graphics.drawString(NO_DATA_MSG, (getWidth() - CellFont.stringWidth(NO_DATA_MSG)) / 2,
                                (getHeight() - LabelHeight - TitleHeight) / 2 + RowHeight, Graphics.BOTTOM | Graphics.LEFT);
            graphics.setClip(old_clip_x, old_clip_y, old_clip_w, old_clip_h);
        }
    }

    public Calendar GetMonth() {
        return Month;
    }

    public int GetSelectedIndex() {
        if (Rows > 0) {
            return SelectedRow;
        } else {
            return -1;
        }
    }
}
