import java.util.*;
import javax.microedition.rms.*;

public class WalletStorageClass extends Object {
    private static final String STORAGE_NAME = "WALLET_STORAGE";

    private static String ValidateErrorMsg;

    private static Hashtable Hash = new Hashtable();

    private static void SortByName(Vector vector, int left, int right) {
        if (right > left) {
            int i = left - 1;
            int j = right;

            while (true) {
                while (((WalletClass)vector.elementAt(++i)).GetName().compareTo(((WalletClass)vector.elementAt(right)).GetName()) < 0);
                while (j > 0) {
                    if (((WalletClass)vector.elementAt(--j)).GetName().compareTo(((WalletClass)vector.elementAt(right)).GetName()) <= 0) {
                        break;
                    }
                }
                if (i >= j) {
                    break;
                }

                WalletClass item = (WalletClass)vector.elementAt(i);

                vector.setElementAt(vector.elementAt(j), i);
                vector.setElementAt(item, j);
            }

            WalletClass item = (WalletClass)vector.elementAt(i);

            vector.setElementAt(vector.elementAt(right), i);
            vector.setElementAt(item, right);

            SortByName(vector, left,  i - 1);
            SortByName(vector, i + 1, right);
        }
    }

    public static void Initialize() throws Exception {
        int         id;
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage                    = RecordStore.openRecordStore(STORAGE_NAME, true);
            RecordEnumeration rec_enum = storage.enumerateRecords(null, null, false);

            while(rec_enum.hasNextElement()) {
                id = rec_enum.nextRecordId();

                Hash.put(new Integer(id), new WalletClass(id, storage.getRecord(id)));
            }
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        }
    }

    public static boolean Validate(WalletClass wallet) {
        if (wallet.GetName().equals("")) {
            ValidateErrorMsg = "Wallet name is empty";

            return false;
        } else {
            Enumeration hash_enum = Hash.elements();

            while (hash_enum.hasMoreElements()) {
                if (wallet.GetName().equals(((WalletClass)hash_enum.nextElement()).GetName())) {
                    ValidateErrorMsg = "Duplicate wallet name";

                    return false;
                }
            }

            return true;
        }
    }

    public static String GetValidateErrorMsg() {
        return ValidateErrorMsg;
    }

    public static WalletClass Get(int id) {
        return (WalletClass)Hash.get(new Integer(id));
    }

    public static void Add(WalletClass wallet) throws Exception {
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage = RecordStore.openRecordStore(STORAGE_NAME, true);

            byte byte_array[] = wallet.ToByteArray();

            int id = storage.addRecord(byte_array, 0, byte_array.length);

            wallet.SetId(id);

            Hash.put(new Integer(id), wallet);
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        }
    }

    public static void Delete(int id) throws Exception {
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage = RecordStore.openRecordStore(STORAGE_NAME, false);
            storage.deleteRecord(id);

            Hash.remove(new Integer(id));
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        }
    }

    public static void Replace(int id, WalletClass wallet) throws Exception {
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage = RecordStore.openRecordStore(STORAGE_NAME, false);

            byte byte_array[] = wallet.ToByteArray();

            storage.setRecord(id, byte_array, 0, byte_array.length);

            wallet.SetId(id);

            Hash.put(new Integer(id), wallet);
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        }
    }

    public static int GetCount() {
        return Hash.size();
    }

    public static WalletClass SearchByName(String name) {
        Enumeration hash_enum = Hash.elements();

        while (hash_enum.hasMoreElements()) {
            WalletClass wallet = (WalletClass)hash_enum.nextElement();

            if (name.equals(wallet.GetName())) {
                return wallet;
            }
        }

        return null;
    }

    public static Vector GetListSortedByName() {
        Vector      result = new Vector();
        Enumeration hash_enum = Hash.elements();

        while (hash_enum.hasMoreElements()) {
            result.addElement(hash_enum.nextElement());
        }

        SortByName(result, 0, result.size() - 1);

        return result;
    }
}
